@@ -16,7 +16,9 @@ import scp
 from archiver import *
 import cron
 
-GHETTO_VERSION='04/24/2010'
+GHETTO_VERSION='05/12/2010'
+
+exit_code_re=re.compile('^\s*exit_code\s*=\s*([0-9]+)\s*$')
 
 # ---------------------------------------------------------------------------
 class MySCPClient(scp.SCPClient):
@@ -63,7 +65,7 @@ class MySCPClient(scp.SCPClient):
             path = os.path.join(self._recv_dir, parts[2])
         except:
             self.channel.send('\x01')
-            raise SCPException('Bad directory format')
+            raise scp.SCPException('Bad directory format')
         else:
             if self._recv_dir==self._recv_topdir and (not os.path.exists(self._recv_dir) or os.path.isdir(self._recv_dir)):
                 # use self._recv_dir instead of os.path.join(self._recv_dir, parts[2])
@@ -76,11 +78,11 @@ class MySCPClient(scp.SCPClient):
             elif os.path.isdir(path):
                 os.chmod(path, mode)
             else:
-                raise SCPException('%s: Not a directory' % path)
+                raise scp.SCPException('%s: Not a directory' % path)
             self._dirtimes[path] = (self._utime)
             self._utime = None
             self._recv_dir = path
-        except (OSError, SCPException), e:
+        except (OSError, scp.SCPException), e:
             self.channel.send('\x01'+e.message)
             raise
         
@@ -90,7 +92,8 @@ def RunCommand(t, command, log=None):
     chan=t.open_session()
     exit_code=None
     chan.set_combine_stderr(True)
-    chan.exec_command(command)
+    cmd_line=command+' ; echo exit_code=$?' # ESXi 3.5 don't return the exit_code   
+    chan.exec_command(cmd_line)
     output=''
     while 1:
         try:
@@ -99,20 +102,26 @@ def RunCommand(t, command, log=None):
                 break
             output+=x
         except socket.timeout:
-            status=-1
             break
 
-    if exit_code==None:
-        exit_code=chan.recv_exit_status()
-        chan.close()
-        
+    exit_code=chan.recv_exit_status()
+    chan.close()
+    
+    if exit_code==-1:
+        # ESXi 3.5 don't return the exit_code   
+        for line in output.split('\n')[-2:]:
+            match=exit_code_re.match(line)
+            if match:
+                exit_code=int(match.group(1))
+                break
+
     if log:
         l=log.debug
         if exit_code!=0:
             l=log.warning
             
         if output or exit_code!=0:
-            l('code=%d command=%s', exit_code, command)
+            l('code=%d command=%s', exit_code, cmd_line)
             for line in output.split('\n'):
                 l('> %s', line)
         
@@ -208,6 +217,8 @@ class Ghettovcb(Archiver):
                 errors[name]='option mandatory'
             else:
                 setattr(self, name, value)
+                if name=='password':
+                    self.register_password(value)
 
         try:
             self.vm_list=quoted_string_list(job.get('vm_list', ''))
@@ -262,7 +273,7 @@ class Ghettovcb(Archiver):
                 
                 
             if self.default_var.get('LAST_MODIFIED_DATE','')!=GHETTO_VERSION:
-                warning['local']='ghettoVCB.sh version is different, use the one provided with MKBackup or be warned'
+                warnings['local']='ghettoVCB.sh version is different, use the one provided with MKBackup or be warned'
             
             
             # read and extract gloabl_conf
@@ -280,6 +291,9 @@ class Ghettovcb(Archiver):
             
         self.destination=job.get('destination', None)
         if self.destination:
+            if not self.vm_list:
+                 warnings['destination']='vm_list must be set to use "destination"'
+                 
             self.destination=self.destination.replace('\n','')
             try:
                 self.destination=Destinations(self.destination, self)
@@ -297,9 +311,12 @@ class Ghettovcb(Archiver):
                 if not os.path.isfile(self.scp_bin):
                     errors['scp_bin']='file not found'
 
-
+        if not errors and self.destination:
+            self.type, self.target=self.destination.match(now, self.night_shift, variables=dict(vm=self.vm_list[0]))
+            if self.type in ('move', 'copy') and not os.path.isdir(os.path.dirname(self.target)):
+                errors['destination']='directory not found: %s' % (os.path.dirname(self.target), )
+                    
         if not errors:
-
             # upload and setup files on the VMWARE server
 
             t=paramiko.Transport((self.host, self.port))
@@ -329,7 +346,9 @@ class Ghettovcb(Archiver):
                         else:
                             _exit_code, _output=RunCommand(t, 'echo "%s" >> "%s"' % (vm, self.exclude_vm_file), log)
             if self.global_conf:
-                self.cmd_line+=' -g "%s"' % (self.global_conf,)       
+                self.cmd_line+=' -g "%s"' % (posixpath.join(self.temp_target, self.global_conf),)
+            
+            self.cmd_line+=' ; echo exit_code=$?' # ESXi 3.5 don't return the exit_code   
 
             extra+='cmd_line=%s\n' % self.cmd_line
             t.close()
@@ -338,8 +357,8 @@ class Ghettovcb(Archiver):
                 self.scp_args+=[ '-P', str(self.port), '-q' ]
                 if sys.platform in ('win32', ):
                     # putty allow cmd line password
-                    self.scp_args+=[ '-batch', '-pw', '<password>' ]
-                extra+='scp_cmd=%s %s@%s:%%source%% %%destination%%\n' % (' '.join(self.scp_args), self.login, self.host, ) 
+                    self.scp_args+=[ '-batch', '-pw', self.password ]
+                extra+='scp_cmd=%s %s@%s:%%source%% %%destination%%\n' % (self.hide_password(' '.join(self.scp_args)), self.login, self.host, ) 
                 
         return errors, warnings, extra
 
@@ -361,6 +380,7 @@ class Ghettovcb(Archiver):
             result=self.sub_run(command, job, manager)
 
             code=send_mail_report(job, result, manager)
+            manager.start_logging()
             status=max(status, code)
 
         job['name']=name
@@ -370,6 +390,8 @@ class Ghettovcb(Archiver):
     # -----------------------------------------------------------------------
     def sub_run(self, command, job, manager):
 
+        no_error=True
+
         log=manager.log
         now=manager.now
 
@@ -378,7 +400,7 @@ class Ghettovcb(Archiver):
         t=paramiko.Transport((self.host, self.port))
         t.get_security_options()._set_ciphers(['blowfish-cbc', ])
         t.connect(username=self.login, password=self.password)
-        t.set_keepalive(300)
+        # t.set_keepalive(300) # make the backup fail
         if self.current_vm:
             # create the current_vm_file
             _exit_code, _output=RunCommand(t, 'echo "%s" > "%s"' % (self.current_vm, self.current_vm_file), log)
@@ -404,15 +426,26 @@ class Ghettovcb(Archiver):
         ghettovcb_exit_code=chan.recv_exit_status()
         chan.close()
         end=int(time.time())
-
+        
+        for line in output.split('\n')[-2:]:
+            match=exit_code_re.match(line)
+            if match:
+                candidate=int(match.group(1))
+                if ghettovcb_exit_code==-1:
+                    ghettovcb_exit_code=candidate
+                elif ghettovcb_exit_code!=candidate:
+                    log.warning('ghettovcb exit_code mismatch ssh=%d log=%d', ghettovcb_exit_code, candidate)
+                    # use the one from SSH anyway
+                break
+                    
         # search the target and make a directory listing 
-        dir_lst, df, scp_process, target=None, None, None, None
+        dir_lst, up_lst, df, scp_process, target=None, None, None, None, None
         if self.current_vm and self.vm_var[self.current_vm]['ENABLE_NON_PERSISTENT_NFS']!=1:
             target=posixpath.join(self.vm_var[self.current_vm]['VM_BACKUP_VOLUME'], self.current_vm)
             compress=self.vm_var[self.current_vm]['ENABLE_COMPRESSION']=='1'
-            posixpath.join(target, self.current_vm)
             if compress:
                 _exit_code, dir_lst=RunCommand(t, 'ls -l "%s"' % target, log)
+                dir_lst='== directory '+target+'\n'+dir_lst
             else:
                 _exit_code, up_lst=RunCommand(t, 'ls "%s"' % target, log)
                 dir_lst=None
@@ -422,62 +455,62 @@ class Ghettovcb(Archiver):
                     if line.startswith(self.current_vm):
                         if line>target_name:
                            target_name=line
+                up_lst='== directory '+target+'\n'+up_lst
                 if target_name: 
                     target=posixpath.join(target, target_name)
                     _exit_code, dir_lst=RunCommand(t, 'ls -l "%s"' % target, log)
-    
+                    dir_lst='== directory '+target+'\n'+dir_lst
             # if destination, move or copy backup
             if self.destination:
                 self.type, self.target=self.destination.match(now, self.night_shift, variables=dict(vm=self.current_vm))
                 if self.type in ('move', 'copy'):
-                    
                     if not compress:
-                        # create the target dir, if dont exist
+                        # create the target dir, if don't exist
                         try:
-                            os.mkdir(self.target)
-                        except OSError, e:
-                            log.debug('creating directory %s: %s', self.target, e)
-                        except:
+                            if not os.path.isdir(self.target):
+                                os.makedirs(self.target)
+                        except Exception, e:
                             log.error('creating directory %s: %s', self.target, e)
+                            no_error=False
     
-                    if not self.scp_bin:
-                        # use paramiko
-                        scpclient=MySCPClient(t, buff_size=65536)
-                        log.info('scp -r %s %s', target, self.target)
-                        scpclient.get(target, self.target, recursive=True)
-                    else:
-                        # use local SCP
-                        scp_args=self.scp_args[:]
-                        if not compress:
-                            if sys.platform in ('win32', ):
-                                scp_args.append('-unsafe')
-                            #scp_args.append('-r')
-                            wildcard='/*'
+                    if no_error:
+                        if not self.scp_bin:
+                            # use paramiko
+                            scpclient=MySCPClient(t, buff_size=65536)
+                            log.info('scp -r %s %s', target, self.target)
+                            scpclient.get(target, self.target, recursive=True)
                         else:
-                            wildcard=''
+                            # use local SCP
+                            scp_args=self.scp_args[:]
+                            if not compress:
+                                if sys.platform in ('win32', ):
+                                    scp_args.append('-unsafe')
+                                #scp_args.append('-r')
+                                wildcard='/*'
+                            else:
+                                wildcard=''
+                                
+                            scp_args+=[ '%s@%s:%s%s' % (self.login, self.host, target, wildcard), self.target]
+                            scp_cmd=self.hide_password(' '.join(scp_args))
+                            log.info('scp_cmd=%s', scp_cmd)
+                            if sys.platform in ('win32', ):
+                                scp_args=param_encode(scp_args, manager.default_encoding)
+                            scp_process=subprocess.Popen(scp_args, executable=self.scp_bin, stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+                            scp_out, scp_err=scp_process.communicate()
                             
-                        scp_args+=[ '%s@%s:%s%s' % (self.login, self.host, target, wildcard), self.target]
-                        scp_cmd=' '.join(scp_args)
-                        log.info('scp_cmd=%s', scp_cmd)
-                        mypass={ '<password>' : self.password}
-                        scp_args=map(lambda x: mypass.get(x, x), scp_args)
-                        scp_process=subprocess.Popen(scp_args, executable=self.scp_bin, stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
-                        scp_out, scp_err=scp_process.communicate()
-    
-                    log.info('scp end')
+                        log.info('scp end')
                         
-                        
-                if self.type=='move':
-                    # delete the source
-                    log.info('delete %s', target)
-                    _exit_code, dir_lst=RunCommand(t, 'rm -r "%s"' % target, log)
+                        if self.type=='move':
+                            # delete the source
+                            log.info('delete %s', target)
+                            _exit_code, _rm_out=RunCommand(t, 'rm -r "%s"' % target, log)
     
             # make the disk space usage
             _exit_code, df=RunCommand(t, 'df', log)
 
         t.close()
 
-        if ghettovcb_exit_code==0 and (not scp_process or scp_process.returncode==0):
+        if ghettovcb_exit_code==0 and (not scp_process or scp_process.returncode==0) and no_error:
             backup_status='OK'
         else:
             backup_status='ERR'
@@ -492,16 +525,12 @@ class Ghettovcb(Archiver):
         status+='exit_code=%d\r\n' % ghettovcb_exit_code
         status+='start=%s\r\n' % time.ctime(start)
         status+='end=%s\r\n' % time.ctime(end)
-        if target:
-            status+='remote_target=%s\r\n' % target
+        status+='start_epoch=%d\r\n' % start
+        status+='end_epoch=%d\r\n' % end
 
         status+='type=%s\r\n' % self.type
         status+='target=%s\r\n' % self.target
-        if self.target and scp_process:
-            status+='scp_exit_code=%d\n' % scp_process.returncode
             
-        status+='start_epoch=%d\r\n' % start
-        status+='end_epoch=%d\r\n' % end
 
         attachements=[ # the type, subtype and coding is not used
                        ('output.txt', None, output, 'text', 'plain', 'utf-8'),
@@ -509,14 +538,31 @@ class Ghettovcb(Archiver):
         if dir_lst:
             attachements.append(('dir.txt', None, dir_lst, 'text', 'plain', 'utf-8'))
 
+        if up_lst:
+            attachements.append(('dir_up.txt', None, up_lst, 'text', 'plain', 'utf-8'))
+
         if df:
             attachements.append(('df.txt', None, df, 'text', 'plain', 'utf-8'),)
 
-        if scp_process and scp_out:
-            attachements.append(('scp_out.txt', None, scp_out, 'text', 'plain', 'cp850'))
-        if scp_process and scp_err:
-            attachements.append(('scp_err.txt', None, scp_err, 'text', 'plain', 'cp850'))
+        if scp_process:
+            status+='scp_cmd=%s\r\n' % scp_cmd
+            status+='scp_exit_code=%d\r\n' % scp_process.returncode
+            status+='remote_target=%s\r\n' % target
+            if scp_out:
+                attachements.append(('scp_out.txt', None, scp_out, 'text', 'plain', manager.console_encoding))
+            if scp_err:
+                attachements.append(('scp_err.txt', None, scp_err, 'text', 'plain', manager.console_encoding))
+
+            try:
+                total_free_bytes=free_space(self.target, log)
+            except Exception, e:
+                log.error('checking target directory: %s', e)
+                status+='target_free_space=%s\r\n' % e
+            else:
+                status+='target_free_space=%d\r\n' % total_free_bytes
+
+            dir_out, _total_size=list_dir(self.target, log)        
+            attachements.append(('target_dir.txt', None, dir_out.encode('utf-8'), 'text', 'plain', 'utf-8'))
 
         
         return backup_status, status, attachements
-
